/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SETTINGS_H
#define SETTINGS_H

#include <string>
#include <vector>
#include <map>
#include "SDL.h"

namespace Settings
{
	struct Graphics
	{
		Graphics();
		int scaler;
		int fullscreen;
		int fullscreenWidth;
		int fullscreenHeight;
		int stretch;
		int keepAspectRatio;
		std::string pointer;
		int scalePointer;
		
		void save(std::ostream&);
		void load(std::istream&);
	};
	
	struct Players
	{
		Players();
		std::string humanNames[8];
		std::string computerNames[8];
		int computerDifficulties[8];
		
		void save(std::ostream&);
		void load(std::istream&);
	};
	
	struct Network
	{
		Network();
		int players;
		int network;
		std::string name;
		int maxPlayers;
		int maxPlayersPerClient;
		int computerPlayers;
		int computerDifficulty;
		int laps;
		int tracks;
		int collisions;
		bool tournament;
		
		std::vector<Uint16> internetPorts;
		std::vector<Uint16> lanPorts;
		std::string server;
		
		unsigned maxConnections;
		int chatFont;
		unsigned maxChatMessages;
		int chatMessageDuration;
		
		void save(std::ostream&);
		void load(std::istream&);
	};
	
	struct Tracks
	{
		Tracks();
		int minLength;
		int maxLength;
		int minWidth;
		int maxWidth;
		int minHeight;
		int maxHeight;
		int minCorners;
		int maxCorners;
		int minLongestStraighaway;
		int maxLongestStraighaway;
		int minLongestCornerSeries;
		int maxLongestCornerSeries;
		
		std::map<std::string, int> tilesets;
		
		void save(std::ostream&);
		void load(std::istream&);
	};
	
	struct Race
	{
		Race();
		int humanPlayers;
		int computerPlayers;
		int laps;
		int tracks;
		int collisions;
		int difficulty;
		bool tournament;
		
		void save(std::ostream&);
		void load(std::istream&);
	};
	
	struct TimeTrial
	{
		TimeTrial();
		std::string name;
		int car;
		int remember;
		int showDriverGhost;
		int showBestGhost;
		int laps;
		
		void save(std::ostream&);
		void load(std::istream&);
	};
	
	extern Graphics graphics;
	extern Players players;
	extern Network network;
	extern Tracks tracks;
	extern Race race;
	extern TimeTrial timetrial;
	
	void save(std::ostream&);
	void load(std::istream&);
}

const int MIN_TRACK_LENGTH = 6;
const int MAX_TRACK_LENGTH = 70;
const int MIN_TRACK_WIDTH = 2;
const int MAX_TRACK_WIDTH = 10;
const int MIN_TRACK_HEIGHT = 2;
const int MAX_TRACK_HEIGHT = 7;
const int MIN_TRACK_CORNERS = 0;
const int MAX_TRACK_CORNERS = 100;
const int MIN_LONGEST_STRAIGHTAWAY = 1;
const int MAX_LONGEST_STRAIGHTAWAY = 8;
const int MIN_LONGEST_CORNER_SERIES = 1;
const int MAX_LONGEST_CORNER_SERIES = 70;

const int DEFAULT_LAN_PORTS_FIRST = 6300;
const int DEFAULT_LAN_PORTS_LAST  = 6302;

#endif
